#include "iogateway/PlainTextMessageIOGateway.h"

AbstractMessageIOGateway * PlainTextMessageIOGatewayFactoryFunc(PortableDataIO * io)
{
   return newnothrow PlainTextMessageIOGateway(io);
}


PlainTextMessageIOGateway ::
PlainTextMessageIOGateway(PortableDataIO * io) : AbstractMessageIOGateway(io), _eolString("\r\n"), _prevCharWasCarriageReturn(false)
{
   // empty
}

PlainTextMessageIOGateway ::
~PlainTextMessageIOGateway()
{
   // empty
}

status_t 
PlainTextMessageIOGateway ::
DoOutput(uint32 & wroteBytes)
{
   const PortableMessage * msg = _currentSendingMessage.GetItemPointer();
   if (msg == NULL) 
   {
      // try to get the next message from our queue
      (void) GetOutgoingMessageQueue().RemoveHead(_currentSendingMessage);
      msg = _currentSendingMessage.GetItemPointer();
      _currentSendLineIndex = _currentSendOffset = -1;
   }
   if (msg)
   {
      if ((_currentSendOffset < 0)||(_currentSendOffset >= (int32)_currentSendText.Length()))
      {
         // Try to get the next line of text from our message
         if (msg->FindString(PR_NAME_TEXT_LINE, ++_currentSendLineIndex, _currentSendText) == B_NO_ERROR) 
         {
            _currentSendOffset = 0;
            _currentSendText += _eolString;
         }
         else 
         {
            _currentSendingMessage.Reset();  // no more text available?  Go to the next message then.
            return DoOutput(wroteBytes);
         }
      }
      if ((msg)&&(_currentSendOffset >= 0)&&(_currentSendOffset < (int32)_currentSendText.Length()))
      {
         // Send as much as we can of the current text line
         const char * bytes = _currentSendText.Cstr() + _currentSendOffset;
         ssize_t bytesWritten = GetDataIO()->Write(bytes, _currentSendText.Length() - _currentSendOffset);
         if (bytesWritten < 0) 
         {
            FlushPendingInput();
            return B_ERROR;
         }
         else if (bytesWritten > 0)
         {
            _currentSendOffset += bytesWritten;
            wroteBytes += bytesWritten;
            return DoOutput(wroteBytes);
         }
      }
   }
   return B_NO_ERROR;
}


void
PlainTextMessageIOGateway ::
FlushPendingInput()
{
   if (_incomingText.Length() > 0)
   {
      PortableMessage * inMsg = GetIOMessagePool()->GetObject();
      if (inMsg)
      {
         inMsg->what = PR_COMMAND_TEXT_STRINGS;
         (void) inMsg->AddString(PR_NAME_TEXT_LINE, _incomingText);
         _incomingText = "";
         (void) GetIncomingMessageQueue().AddTail(PortableMessageRef(inMsg, GetIOMessagePool()));
      }
   }
}

status_t 
PlainTextMessageIOGateway ::
DoInput(uint32 & setReadBytes)
{
   const int tempBufSize = 2048;
   char buf[tempBufSize];
   ssize_t bytesRead = GetDataIO()->Read(buf, sizeof(buf));
   if (bytesRead < 0) 
   {
      FlushPendingInput();
      return B_ERROR;
   }
   if (bytesRead > 0)
   {
      setReadBytes += bytesRead;
      PortableMessage * inMsg = NULL;  // demand-allocated
      int32 beginAt = 0;
      for (int32 i=0; i<bytesRead; i++)
      {
         char nextChar = buf[i];
         if ((nextChar == '\r')||(nextChar == '\n'))
         {
            buf[i] = '\0';  // terminate the string here
            if ((nextChar == '\r')||(_prevCharWasCarriageReturn == false)) 
            {
               if (inMsg == NULL) inMsg = GetIOMessagePool()->GetObject();
               if (inMsg)
               {
                  inMsg->what = PR_COMMAND_TEXT_STRINGS;
                  if (_incomingText.Length() > 0) 
                  {
                     (void) inMsg->AddString(PR_NAME_TEXT_LINE, _incomingText.Append(&buf[beginAt]));
                     _incomingText = "";
                  }
                  else (void) inMsg->AddString(PR_NAME_TEXT_LINE, &buf[beginAt]);
               }
            }
            beginAt = i+1;
         }
         _prevCharWasCarriageReturn = (nextChar == '\r');
      }
      if (beginAt < bytesRead) _incomingText += &buf[beginAt];
      if (inMsg) (void) GetIncomingMessageQueue().AddTail(PortableMessageRef(inMsg, GetIOMessagePool()));
   }
   return B_NO_ERROR;
}

bool 
PlainTextMessageIOGateway ::
HasBytesToOutput() const
{
   return ((_currentSendingMessage.GetItemPointer() != NULL)||(GetOutgoingMessageQueue().CountItems() > 0));
}
