/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.txt file for details. */

#ifndef PLAIN_TEXT_IO_GATEWAY
#define PLAIN_TEXT_IO_GATEWAY

#include "dataio/PortableDataIO.h"
#include "message/PortableMessage.h"
#include "util/PortableRefCount.h"
#include "iogateway/AbstractMessageIOGateway.h"

/** This is the name of the string field used to hold text lines. */
#define PR_NAME_TEXT_LINE "tl"

/** The 'what' code that will be found in incoming PortableMessages. */
#define PR_COMMAND_TEXT_STRINGS 1886681204 // 'ptxt'

/** Returns a new PlainTextMessageIOGateway object using the given (io), or NULL on failure. */
AbstractMessageIOGateway * PlainTextMessageIOGatewayFactoryFunc(PortableDataIO * io);

/** 
 * This gateway translates lines of text (separated by \r, \n, or \r\n) into
 * PortableMessages.  It can be used for "telnet-style" net interactions.
 * Incoming and outgoing messages may have one or more strings in their PR_NAME_TEXT_LINE field.
 * Each of these strings represents a line of text (separator chars not included)
 */
class MUSCLE_CLASS_DECLARATIONS PlainTextMessageIOGateway : public AbstractMessageIOGateway
{
public:
   /** This object becomes the owner of (io) and will delete it later. */
   PlainTextMessageIOGateway(PortableDataIO * io);

   /** Destructor */
   virtual ~PlainTextMessageIOGateway();

   virtual status_t DoOutput(uint32 & retWroteBytes);
   virtual status_t DoInput(uint32 & retReadBytes);
   virtual bool HasBytesToOutput() const;

   /** Set the end-of-line string to be attached to outgoing text lines.
    * Default value is "\r\n".
    */
   virtual void SetOutgoingEndOfLineString(const char * str) {_eolString = str;}

private:
   void FlushPendingInput();

   PortableMessageRef _currentSendingMessage;
   PortableString _currentSendText;
   PortableString _eolString;
   int32 _currentSendLineIndex;
   int32 _currentSendOffset;
   bool _prevCharWasCarriageReturn;
   PortableString _incomingText;
};

#endif
