/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.txt file for details. */

/******************************************************************************
/
/	File:			PortableTypeConstants.h
/
/	Description:	Constants that represent distinct data types, as used by PortableMessage et. al.
/
******************************************************************************/

#ifndef _PORTABLETYPE_CONSTANTS_H
#define _PORTABLETYPE_CONSTANTS_H

#include "support/PortableByteOrder.h"

#define MAKETYPE(x) ((((unsigned long)(x[0])) << 24) | \
                     (((unsigned long)(x[1])) << 16) | \
                     (((unsigned long)(x[2])) <<  8) | \
                     (((unsigned long)(x[3])) <<  0))
#ifdef __BEOS__
#include <support/TypeConstants.h>
#else
//:Be-style message-field type codes.
// I've calculated the integer equivalents for these codes
// because gcc whines like a little girl about the four-byte
// constants when compiling under Linux --jaf
enum {
   B_ANY_TYPE 	  = 1095653716, // 'ANYT',  // wild card
   B_BOOL_TYPE 	  = 1112493900, // 'BOOL',
   B_DOUBLE_TYPE  = 1145195589, // 'DBLE',
   B_FLOAT_TYPE   = 1179406164, // 'FLOT',
   B_INT64_TYPE   = 1280069191, // 'LLNG',
   B_INT32_TYPE   = 1280265799, // 'LONG',
   B_INT16_TYPE   = 1397248596, // 'SHRT',
   B_INT8_TYPE 	  = 1113150533, // 'BYTE',
   B_MESSAGE_TYPE = 1297303367, // 'MSGG',
   B_POINTER_TYPE = 1347310674, // 'PNTR',
   B_POINT_TYPE   = 1112559188, // 'BPNT',
   B_RECT_TYPE    = 1380270932, // 'RECT',
   B_STRING_TYPE  = 1129534546, // 'CSTR',
   B_OBJECT_TYPE  = 1330664530, // 'OPTR',  // used for flattened objects
   B_RAW_TYPE     = 1380013908, // 'RAWT',  // used for raw byte arrays
   B_MIME_TYPE    = 1296649541  // 'MIME',  // new for v1.44
};
#endif

#include "support/PortableByteOrder.h"

//:Macro to turn a type_code into a string representation.
// (typecode) is the type_code to get the string for
// (buf) is a (char *) to hold the output string; it must be >= 5 bytes long.
#define MakePrettyTypeCodeString(typecode, buf)                     \
   {                                                                \
      uint32 __bigEndian = B_HOST_TO_BENDIAN_INT32(typecode);       \
      memcpy(buf, (const char *)&__bigEndian, sizeof(__bigEndian)); \
      buf[sizeof(__bigEndian)] = '\0';                              \
   }           

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

#endif /* _PORTABLETYPE_CONSTANTS_H */
