/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>
#include <stdarg.h>

#include "util/PortableHashtable.h"
#include "util/PortableString.h"
#include "util/StringTokenizer.h"

int _state = 0;

static void bomb(const char * fmt, ...);
void bomb(const char * fmt, ...)
{
   va_list va;
   va_start(va, fmt);
   vprintf(fmt, va);
   va_end(va);
   printf("EXITING DUE TO ERROR (state = %i)!\n", _state);
   exit(10);
}

uint32 SillyHashFunc(const uint32 & val);
uint32 SillyHashFunc(const uint32 & val)
{
   return val;
}

uint32 IntHashFunc(const int & val);
uint32 IntHashFunc(const int & val)
{
   return val;
}

int MyCompareFunc(const int & s1, const int & s2, void *);
int MyCompareFunc(const int & s1, const int & s2, void *)
{
   return s1-s2;
}

static int DoInteractiveTest();
static int DoInteractiveTest()
{
   PortableHashtable<int, PortableString> table(IntHashFunc);

   // Prepopulate the table, for convenience
   for (int i=9; i>=0; i--)
   {
      char buf[32];
      sprintf(buf, "%i", i);
      table.Put(i, buf);
   }

   // Let's see if this reverses it
   {
      PortableHashtableIterator<int, PortableString> rev = table.GetIterator();
      int next;
      while(rev.GetNextKey(next)) table.MoveToFront(next);
   }

   // turn on auto-sorting
   table.SetCompareFunction(MyCompareFunc, NULL, true);

   while(true)
   {
      PortableHashtableIterator<int, PortableString> iter = table.GetIterator();

      {
         bool first = true;
         printf("\nCurrent contents: ");
         int nextKey;
         while(iter.GetNextKey(nextKey))
         {
            PortableString * nextValue = iter.GetNextValue();
            MASSERT((atoi(nextValue->Cstr()) == nextKey), "value/key mismatch A!\n");
            printf("%s%i", first?"":", ", nextKey);
            first = false;
         }
         printf(" (size=%lu)\nEnter command: ", table.Size()); fflush(stdout);
      }

      char buf[512];
      if (fgets(buf, sizeof(buf), stdin))
      {
         StringTokenizer tok(buf, " ");
         const char * arg0 = tok();
         const char * arg1 = tok(); 
         const char * arg2 = tok(); 

         char command = arg0 ? arg0[0] : '\0';

         // For extra fun, let's put a half-way-done iterator here to see what happens
         printf("Concurrent: ");
         PortableHashtableIterator<int, PortableString> iter(table.GetIterator());
         bool first = true;
         int nextKey;
         if (table.Size() > 0)
         {
            for (int i=table.Size()/2; i>=0; i--) 
            {
               MASSERT(iter.GetNextKey(nextKey), "Not enough keys in table!?!?\n");
               PortableString * nextValue = iter.GetNextValue();
               MASSERT((atoi(nextValue->Cstr()) == nextKey), "value/key mismatch B!\n");

               printf("%s%i", first?"":", ", nextKey);
               first = false;
            }
         }

         switch(command)
         { 
            case 'F':  
               if ((arg1)&&(arg2))
               {
                  printf("%s(%s %i before %i)", first?"":", ", table.MoveToBefore(atoi(arg1),atoi(arg2)) ? "Befored" : "FailedToBefored", atoi(arg1), atoi(arg2));
                  first = false;
               }
               else printf("(No arg1 or arg2!)");
            break;
            
            case 'B':  
               if ((arg1)&&(arg2))
               {
                  printf("%s(%s %i behind %i)", first?"":", ", table.MoveToBehind(atoi(arg1),atoi(arg2)) ? "Behinded" : "FailedToBehind", atoi(arg1), atoi(arg2));
                  first = false;
               }
               else printf("(No arg1 or arg2!)");
            break;
            
            case 'f':  
               if (arg1)
               {
                  printf("%s(%s %i)", first?"":", ", table.MoveToFront(atoi(arg1)) ? "Fronted" : "FailedToFront", atoi(arg1));
                  first = false;
               }
               else printf("(No arg1!)");
            break;
            
            case 'b':  
               if (arg1)
               {
                  printf("%s(%s %i)", first?"":", ", table.MoveToBack(atoi(arg1)) ? "Backed" : "FailedToBack", atoi(arg1));
                  first = false;
               }
               else printf("(No arg1!)");
            break;
            
            case 'p':  
               if (arg1)
               {
                  printf("%s(%s %i)", first?"":", ", table.Put(atoi(arg1), arg1) ? "Put" : "FailedToPut", atoi(arg1));
                  first = false;
               }
               else printf("(No arg1!)");
            break;
            
            case 'r':  
               if (arg1)
               {
                  printf("%s(%s %i)", first?"":", ", table.Remove(atoi(arg1)) ? "Removed" : "FailedToRemove", atoi(arg1));
                  first = false;
               }
               else printf("(No arg1!)");
            break;

            case 'c':  
               printf("%s(Clearing table)", first?"":", ");
               table.Clear();
               first = false;
            break;

            case 's':
               printf("%s(Sorting table)", first?"":", ");
               table.Sort();
               first = false;
            break;

            case 'q':  
               printf("%sQuitting\n", first?"":", ");
               first = false;
               return 0;
            break;
         }

         while(iter.GetNextKey(nextKey))
         {
            PortableString * nextValue = iter.GetNextValue();
            MASSERT((atoi(nextValue->Cstr()) == nextKey), "value/key mismatch C!\n");

            printf("%s%i", first?"":", ", nextKey);
            first = false;
         }
         printf("\n");
      }
      else return 0;
   }
}

// This program exercises the PortableHashtable class.
int main(int argc, char ** argv)
{
   if ((argc > 1)&&(strcmp(argv[1], "inter") == 0)) return DoInteractiveTest();

   PortableHashtable<PortableString, PortableString> table(PortableStringHashFunc);
   {
      table.Put("Hello", "World");
      table.Put("Peanut Butter", "Jelly");
      table.Put("Ham", "Eggs");
      table.Put("Pork", "Beans");
      table.Put("Slash", "Dot");
      table.Put("Data", "Mining");  
      table.Put("TestDouble", "Play");
      table.Put("Abbot", "Costello");
      table.Put("Laurel", "Hardy");
      table.Put("Thick", "Thin");
      table.Put("Butter", "Parkay");
      table.Put("Total", "Carnage");
      table.Put("Summer", "Time");
      table.Put("Terrible", "Twos");

      printf("String table has %i entries in it...\n", table.Size());

      PortableString lookup;
      if (table.Get(PortableString("Hello"), lookup)) printf("Hello -> %s\n", lookup());
                                                 else printf("Lookup 1 failed.\n");
      if (table.Get(PortableString("Peanut Butter"), lookup)) printf("Peanut Butter -> %s\n", lookup());
                                                 else printf("Lookup 2 failed.\n");


      PortableHashtableIterator<PortableString, PortableString> st = table.GetIterator();
      PortableString nextKeyString;
      PortableString nextValueString;

      printf("Testing normal traveral with GetNextKey() and Get()\n");
      while(st.GetNextKey(nextKeyString))
      {
         bool ret = table.Get(nextKeyString, nextValueString);
         printf("t1 = %i %s: %s -> %s\n", st.HasMoreKeys(), ret ? "OK" : "ERROR", nextKeyString(), nextValueString());
      }

      printf("Testing normal traveral with GetNextKey() and GetNextValue()\n");
      st = table.GetIterator();
      while(st.GetNextKey(nextKeyString))
      {
         bool ret = st.GetNextValue(nextValueString);
         printf("t2 = %i %s: %s -> %s\n", st.HasMoreKeys(), ret ? "OK" : "ERROR", nextKeyString(), nextValueString());
      }

      printf("Testing delete-as-you-go traveral\n");
      st = table.GetIterator();
      while(st.GetNextKey(nextKeyString))
      {
         bool ret = st.GetNextValue(nextValueString);
         printf("t3 = %i %s: %s -> %s (tableSize=%lu)\n", st.HasMoreKeys(), ret ? "OK" : "ERROR", nextKeyString(), nextValueString(), table.Size());
         table.Remove(nextKeyString);
#if 0
         {
            PortableHashtableIterator<PortableString,PortableString> st2 = table.GetIterator();
            while(st2.GetNextKey(nextKeyString))
            {
               bool ret = st2.GetNextValue(nextValueString);
               printf("  tx = %i %s: %s -> %s\n", st2.HasMoreKeys(), ret ? "OK" : "ERROR", nextKeyString(), nextValueString());
            }
         }
#endif
      }

      printf("Testing another normal traversal with GetNextKey() and Get(), should be empty\n");
      while(st.GetNextKey(nextKeyString))
      {
         bool ret = table.Get(nextKeyString, nextValueString);
         printf("t1 = %i %s: %s -> %s\n", st.HasMoreKeys(), ret ? "OK" : "ERROR", nextKeyString(), nextValueString());
      }

      PortableHashtable<uint32, const char *> sillyTable(SillyHashFunc);
      sillyTable.Put(15, "Fifteen");
      sillyTable.Put(100, "One Hundred");
      sillyTable.Put(150, "One Hundred and Fifty");
      sillyTable.Put(200, "Two Hundred");

      const char * temp;
      sillyTable.Get(100, temp);
      sillyTable.Get(101, temp); // will fail
      printf("100 -> %s\n",temp);

      printf("Entries in sillyTable:\n");
      PortableHashtableIterator<uint32, const char *> it = sillyTable.GetIterator();

      uint32 nextKey;
      const char * nextValue;
      while(it.GetNextKey(nextKey))
      {
         bool ret = sillyTable.Get(nextKey, nextValue);
         printf("%i %s: %u -> %s\n", it.HasMoreKeys(), ret ? "OK" : "ERROR", nextKey, nextValue);
      }
      printf("XXX %i\n", it.HasMoreKeys());

      printf("... and by value:\n");
      while(it.GetNextValue(nextValue)) printf("%i value: %s\n", it.HasMoreValues(), nextValue);
      printf("YYY %i\n", it.HasMoreValues());
   }
   table.Clear();   

   printf("Begin torture test!\n");
   _state = 4;
   bool fastClear = false;
   while(true)
   {
      PortableHashtable<PortableString, uint32> t(PortableStringHashFunc);
      for (int numEntries=1; numEntries < 100000; numEntries++)
      {
         int half = numEntries/2;
         bool ok = true;

         printf("%i ", numEntries); fflush(stdout);
         _state = 5;
         {
            for(int i=0; i<numEntries; i++)
            {
               char temp[300];
               sprintf(temp, "%i", i);
               if (t.Put(temp, i) == false)
               {
                  printf("Whoops, (hopefully simulated) memory failure!  (Put(%i/%lu) failed) ... recovering\n", i, numEntries);

                  ok = false;
                  numEntries--;  // let's do this one over
                  half = i;    // so the remove code won't freak out about not everything being there
                  break;
               }
            }
         }

         if (ok)
         {
            //printf("Checking that all entries are still there...\n");
            _state = 6;
            {
               if (t.Size() != numEntries) bomb("ERROR, WRONG SIZE %i vs %i!\n", t.Size(), numEntries);
               for (int i=numEntries-1; i>=0; i--)
               {
                  char temp[300];
                  sprintf(temp, "%i", i);
                  uint32 tv;
                  if (t.Get(temp, tv) == false) bomb("ERROR, MISSING KEY [%s]\n", temp);
                  if (tv != i) bomb("ERROR, WRONG KEY %s != %lu!\n", temp, tv);
               }
            }
      
            //printf("Iterating through table...\n");
            _state = 7;
            {
               uint32 count = 0;
               PortableHashtableIterator<PortableString, uint32> iter = t.GetIterator();
               PortableString nextKey;
               while(iter.GetNextKey(nextKey)) 
               {
                  char buf[300];
                  sprintf(buf, "%i", count);
                  if (nextKey != buf) bomb("ERROR:  iteration was wrong, item %lu was [%s] not [%s]!\n", count, nextKey(), buf);

                  uint32 temp;
                  (void) iter.GetNextValue(temp);
                  if (temp != count) bomb("ERROR:  iteration value was wrong, item %lu was %lu not %lu!i!\n", count, temp, count);

                  count++;
               }
            }

            //printf("Removing the second half of the entries...\n");
            _state = 8;
            {
               for (int i=half; i<numEntries; i++)
               {
                  char temp[300];
                  sprintf(temp, "%i", i);
                  uint32 tv;
                  if (t.Remove(temp, tv) == false) bomb("ERROR, MISSING REMOVE KEY [%s] A\n", temp);
                  if (tv != i) bomb("ERROR, REMOVE WAS WRONG VALUE %lu\n", tv);
               }
            }

            //printf("Iterating over only first half...\n");
            _state = 9;
            {
               uint32 sum = 0; 
               for (int i=0; i<half; i++) sum += i;

               uint32 count = 0, checkSum = 0;
               PortableHashtableIterator<PortableString, uint32> iter = t.GetIterator();
               PortableString nextKey;
               while(iter.GetNextKey(nextKey)) 
               {
                  count++;
                  uint32 temp;
                  (void) iter.GetNextValue(temp);
                  checkSum += temp;
               }
               if (count != half) bomb("ERROR: Count mismatch %i vs %i!\n", count, numEntries);
               if (checkSum != sum)     bomb("ERROR: Sum mismatch %i vs %i!\n", sum, checkSum);
            }
         }

         //printf("Clearing Table (%s)\n", fastClear ? "Quickly" : "Slowly"); 
         _state = 10;
         if (fastClear) t.Clear();
         else
         {
            for (int i=0; i<half; i++)
            {
               char temp[300];
               sprintf(temp, "%i", i);
               uint32 tv;
               if (t.Remove(temp, tv) == false) 
               {
                  PortableString blahKey;
                  uint32 blah;
                  PortableHashtableIterator<PortableString, uint32> p = t.GetIterator();
                  while(p.GetNextKey(blahKey))
                  {
                     if (p.GetNextValue(blah)) printf("[%s]->%i ", blahKey(), blah);
                                          else printf("[%s]->(no value!)\n", blahKey());
                  }
                  printf("\n");
                  bomb("ERROR, MISSING REMOVE KEY [%s] (%i/%i) B\n", temp, i, half);
               }
               if (tv != i) bomb("ERROR, REMOVE WAS WRONG VALUE %lu\n", tv);
            }
         }

         PortableHashtableIterator<PortableString, uint32> paranoia = t.GetIterator();
         if (paranoia.GetNextKey()) bomb("ERROR, ITERATOR CONTAINED ITEMS AFTER CLEAR!\n");

         if (t.Size() != 0) bomb("ERROR, SIZE WAS NON-ZERO (%lu) AFTER CLEAR!\n", t.Size());
         fastClear = !fastClear;
      }
      printf("Finished torture test successfully, testing again...\n");
   }
}
